#!/bin/bash

set -e

# Prints message and exits
fail() {
    echo $1
    exit 1
}

# Delete folder, but ask first
confirm_delete_folder() {
    if [ $SINGLE_OUTPUT -eq 0 ]; then
	if [ -d "$1" ]
	then
	    echo ""
	    echo "Output folder alread exists ($1)"
	    echo "The contents will be removed and replaced if you proceed"
	    read -p "Do you want to continue? [y/N]? " -r
	    if [[ $REPLY =~ ^[Yy]$ ]]
	    then
		rm -rf "$1"
	    else
		echo "Aborting!"
		exit 1
	    fi
	fi
    fi
}

# Checks that a package exists in the PKG_CONFIG_PATHkage() {
require_package() {
    pkg="$1"

    if pkg-config --exists "$pkg"
    then
	true # noop
    else
	echo "Error: pkg-config could not find package named $pkg"
	echo "Possible solution: Set up PKG_CONFIG_PATH correctly, or build the missing"
	echo "package using this script"
	exit 1
    fi
}

# Download a package to a specific folder
download_pkg() {
    url="$1"
    destination_folder="$2"
    wget -N --trust-server-names --directory-prefix="$destination_folder" "$url"
}

# Function that based on an input folder sanitizes all *.pc files
# to have all paths relative to the ${prefix} variable.
# This makes it easier to update them should you ever want to
# move them. (Constructed to deal with over use of absolute paths in
# freetype2.pc)
sanitize_pkgconfig_files() {
    (
	dir="$1"
	for PC in $dir/*.pc
	do
	    # Remove quotes
	    sed -i 's/\"//g' $PC
	    # Extract the contents of the prefix= variable in the package config file
	    PREFIX=`grep -o "^prefix=.*" $PC | sed -e "s/prefix=//g"`
	    # Replace all uses of the absolute path everywhere else in the file.
	    # Ie, replace all occurences of $PREFIX in the .pc file with the
            # ${prefix} variable. Except for the line that starts with "prefix="
            # since that needs to have a valid path and will be used as base.
	    sed -i '/^prefix=/!s|'${PREFIX%/}'|${prefix}|g' $PC
	done
    )
}

# Returns where the output should be placed
select_output_dir() {
    if [ $SINGLE_OUTPUT -ne 0 ]; then
	echo $OUTPUT_DIR/${LIBFOLDER_PREFIX}opera-dep
    else
	echo $OUTPUT_DIR/$LIBFOLDER_PREFIX$1
    fi
}

build_mpg123() {
    MPG123_URL=mpg123-1.23.3.tar.bz2
    MPG123_PKG=`echo "$MPG123_URL" | grep -o "mpg123\-.*\.tar\.bz2"`
    MPG123_FOLDER=${MPG123_PKG%.tar.bz2}
    OUTPUT_DIR_TMP=`select_output_dir $MPG123_FOLDER`
    (
	pushd $STAGING_DIR

	tar -xvf $CACHE_DIR/$MPG123_PKG

	cd $MPG123_FOLDER

	./configure $CROSS_ARGS --prefix=$OUTPUT_DIR_TMP --enable-static=no
	make -j $MAKE_THREADS
	confirm_delete_folder $OUTPUT_DIR_TMP
	make install
	sanitize_pkgconfig_files $OUTPUT_DIR_TMP/lib/pkgconfig/
	popd

    )
    export PKG_CONFIG_PATH=$OUTPUT_DIR_TMP/lib/pkgconfig:$PKG_CONFIG_PATH
}

show_help() {
    cat << EOF
This script builds third party libraries based on the CROSS and PATH
environment variables alone.
Example: To cross compile freetype for mipsel you should run these commands:
# export CROSS=mipsel-linux-uclibc-
# export PATH=/path/to/toolchain/bin/:\$PATH
# $0 --mpg123 output/
EOF
}

# This function is a helper for extracting only the ARCHITECTURE, CROSS and PATH variables
# from a target file. It does so by creating a sub shell, source the target file
# in the sub shell, and then echo the commands needed to set these variables.
# By running this function through "eval" you will end up with CROSS and PATH
# set as if you sourced the target file, but no other environment variables
# in the target file (like PKG_CONFIG_PATH etc) are exported.
setup_env_from_target() {
    (
  SILENT=1
	target_file="$1"
	source $target_file
	echo "export PATH=$PATH"
	echo "export CROSS=$CROSS"
	echo "export ARCHITECTURE=$ARCHITECTURE"
    )
}

setup_pkg_config_from_target() {
    (
	target_file="$1"
	source $target_file
	echo "export PKG_CONFIG_PATH=$PKG_CONFIG_PATH"
    )
}

setup_widevine_config_from_target() {
    (
   target_file="$1"
   source $target_file
   echo "export OEMCRYPTO_VERSION=$OEMCRYPTO_VERSION"
   echo "export OEMCRYPTO_LIB=$OEMCRYPTO_LIB"
   echo "export OEMCRYPTO_LDFLAGS=$OEMCRYPTO_LDFLAGS"
   echo "export WIDEVINE_CFLAGS=\"$WIDEVINE_CFLAGS\""
   echo "export WIDEVINE_LDFLAGS=\"$WIDEVINE_LDFLAGS\""
    )
}

#############################################################################
# Script starts here
#############################################################################

RESOURCES="$PWD/resources" # Where look for some resource files
STAGING_DIR="$PWD/staging"    # Temporary folder Where everything will be built
PATCH_DIR="$PWD/patches"      # Where to look for patches to apply to the packages
CACHE_DIR="$PWD/cache"        # Where to cache downloaded archives
BUILD_ARCH=`uname -m`         # Architecture of machine where this script runs


MAKE_THREADS=$(($(grep -c processor /proc/cpuinfo) + 2)) # Number of threads to launch make with (for supported packages)

SHORTOPTS="ht:p:ls"
LONGOPTS="help,mpg123,local,target:,prefix:,single-output"

OPTS=$(getopt -o $SHORTOPTS --long $LONGOPTS -n "${progname:-}" -- "$@")
eval set -- "$OPTS"

SINGLE_OUTPUT=0
TO_BUILD=""
TARGET=""
LIBFOLDER_PREFIX=""
LOCAL=""
while [ $# -gt 0 ]; do
    case $1 in
        -h|--help) show_help; exit 0;;
        --mpg123) TO_BUILD+="mpg123 "; shift ;;
        -l|--local) LOCAL="YES" ; shift ;;
        -t|--target) TARGET=$2; shift 2 ;;
        -p|--prefix) LIBFOLDER_PREFIX=${2:-}-; shift 2;;
        -s|--single-output) SINGLE_OUTPUT=1; shift ;;
        --) shift; break;;
        *) echo "Error: unknown command line arg: $1" 1>&2; exit 1;;
    esac
done

if [ -z "$TO_BUILD" ]
then
    show_help
    exit 1
fi

if [ "$TARGET" == "local" ]
then
    unset TARGET
    LOCAL="YES"
fi

if [ -n "$TARGET" ]
then
    STAGING_DIR="${STAGING_DIR}-$TARGET"
    TARGET=../targets/${TARGET}.sh
    if [ ! -e $TARGET ]
    then
	echo "Error: $TARGET does not exist"
	exit 1
    fi
    # eval is needed since the function runs in a sub shell
    eval `setup_env_from_target $TARGET`
fi

if [ ! -n "$CROSS" ] && [ ! -n "$LOCAL" ]
then
    echo "CROSS is not set, for local builds use explicit -l or --local argument"
    exit 1
fi

# Build result will be put in $OUTPUT_DIR
OUTPUT_DIR="${1:-}"
if [ -z $OUTPUT_DIR ]
then
    echo "Error: No output directory specified!"
    show_help
    exit 1
fi

# Setup cross compilation based on the CROSS environment variable
CROSS_ARGS=""
if [ -n "$CROSS" ]
then
    echo "Cross compiling"
    HOST=${CROSS%-} # Drop trailing '-'
    # Specify a value in --build that is different from $HOST to
    # trigger cross compilation mode. Prevents some tools to be
    # run during installation phases, like preventing generation of
    # font caches from fonts on the build machine.
    CROSS_ARGS="--target=$HOST --host=$HOST --build=$BUILD_ARCH-dummy-linux-gnu"
    export PKG_CONFIG_LIBDIR= # Prevent pkg-config from finding system libraries
else
    echo "Local build"
fi

# Remove staging folder before building
rm -rf "$STAGING_DIR"

# Create temporary folders
mkdir -p $CACHE_DIR
mkdir -p $STAGING_DIR

# Make output path absolute and create it
mkdir -p $OUTPUT_DIR
pushd $OUTPUT_DIR
OUTPUT_DIR=`pwd`
popd


# Build the packages, the order is important, since early built
# packages can be fed to later packages using PKG_CONFIG_PATH
# variable.
if [[ $TO_BUILD == *mpg123* ]]
then
    build_mpg123
fi
# Remove staging dir
rm -rf "$STAGING_DIR"

# Done!
echo ""
echo "Output available in: $OUTPUT_DIR"
echo ""
